/* ************************************************************************** */
/*
 * 
 * Project: Bluetooth helper

 * Author: Jack Plumb and Greg Flynn
 * Adapted from code by Brandon Smith and Nick Falco
 * Who adapted the code from example code from redbear
 * 
 * Description: Example master device for redbear ble mini running the HCI firmware

 */
/* ************************************************************************** */

#include "config.h"
// threading library
#include "plib.h"
// config.h sets 40 MHz
#define	SYS_FREQ 40000000
#include "pt_cornell_1_2.h"
#include "xc.h"
#include <stdlib.h>

// BLE library
#include "biscuit_central.h"
#include "ble_hci.h"

#define PLAYER0 0x0000
#define PLAYER1 0x0001
#define PLAYER2 0x0002

#define SS      LATBbits.LATB13
#define dirSS   TRISBbits.TRISB13
#define anSS    ANSELBbits.ANSB13

//uint8_t player1_address[6] = {0x23, 0x11, 0x7B, 0x4C, 0x99, 0xB4}; // Red Bear
//uint8_t player2_address[6] = {0xBA, 0x29, 0x7B, 0x4C, 0x99, 0xB4}; // Red Bear
//uint8_t player3_address[6] = {0x56, 0x19, 0x7B, 0x4c, 0x99, 0xB4}; // Red Bear: B4:99:4C:7B:19:56

uint8_t green_address[6] = {0x74, 0x03, 0x7B, 0x4C, 0x99, 0xB4};
uint8_t blue_address[6] = {0x10, 0x0E, 0x7B, 0x4C, 0x99, 0xB4};
uint8_t yellow_address[6] = {0x6A, 0x03, 0x7B, 0x4C, 0x99, 0xB4};
uint8_t orange_address[6] = {0x6E, 0x19, 0x7B, 0x4C, 0x99, 0xB4};

static struct pt pt_sendBytes, pt_listenByte, pt_input;

uint8_t writeBuf[5]; 
//Make sure this is the same size as the packet you are sending
//Transmitting extra null characters can mess you up


static PT_THREAD (protothread_sendBytes(struct pt *pt))
{
    PT_BEGIN(pt);
    while(1) {
        PT_YIELD_TIME_msec(1000);
        
        sprintf( writeBuf, "N N E" ); //this is the command format I was using
        
//        sprintf( writeBuf, "1 2 j 3");
        //This is the command format the people before me used
        
        biscuit_central_write_bytes(PLAYER0, writeBuf, sizeof(writeBuf));
        PT_YIELD_TIME_msec(10000);
        
//        sprintf( writeBuf, "4 5 j 6" ); 
//        biscuit_central_write_bytes(PLAYER1, writeBuf, sizeof(writeBuf));
//        PT_YIELD_TIME_msec(1000);
//        
//        sprintf( writeBuf, "7 8 j 9" ); 
//        biscuit_central_write_bytes(PLAYER2, writeBuf, sizeof(writeBuf));
//        PT_YIELD_TIME_msec(1000);
//        
    }
    PT_END(pt);
}

//delays durin init
void delay( unsigned t)
{ //uses internal timer 2
    T2CON = 0x8000;     // enable tmr1, Tcy, 1:1
    while (t--)
    {
        TMR2 = 0;
        while( TMR2< 8000);

    }
}


// === Main  ======================================================
void main(void) {
//    SYSTEMConfigPerformance(PBCLK);

    ANSELA = 0; ANSELB = 0; CM1CON = 0; CM2CON = 0;

    // === config threads ==========
    // turns OFF UART support and debugger pin
    PT_setup();

    // === setup system wide interrupts  ========
    INTEnableSystemMultiVectoredInt();
       
    mJTAGPortEnable(0);
    
    mPORTBSetPinsDigitalOut(BIT_15); //reset pin
    
    mPORTBSetBits(BIT_15);
    delay(2500);
    mPORTBClearBits(BIT_15);
    delay(2500);
    
    
    // set up BLE chip
    biscuit_central_init();
    delay(5000);
    
    biscuit_central_connect(blue_address);
    delay(45000);
    
//    biscuit_central_connect(green_address);
//    delay(45000);
//    
//    biscuit_central_connect(orange_address);
//    delay(45000);
    
//    biscuit_central_enable_notification(PLAYER0);
//    delay(2500);
    
        // init the threads
    PT_INIT(&pt_sendBytes);

    // round-robin scheduler for threads
    while (1){
        PT_SCHEDULE(protothread_sendBytes(&pt_sendBytes));
    }
  } // main
/* *****************************************************************************
 End of File
 */
